local addon, private  = ...
local factionPrevious = {}           -- stores factions of previous faction scan for camparing with new notoriety values
local text            = private.text -- language dependent texts

-- global status variables
local locked    = true  -- used to toggle if bar is locked or not
local visible   = false -- used to toggle visibility of the bar; initially all frames are invisible
local mousedown = false -- indicates if left mousebutton is pressed

-- UI elements: faction bar
local factionbar        = {}                                                                                          -- "container" for faction bar
factionbar.context      = UI.CreateContext(addon.identifier .. ".factionbar.context")                                 -- context for the bar
factionbar.frame        = {}                                                                                          -- "container" for faction bar frames
factionbar.frame.bar    = UI.CreateFrame("Frame", addon.identifier .. ".factionbar.frame.bar", factionbar.context)    -- the bar frame itself
factionbar.frame.border = UI.CreateFrame("Frame", addon.identifier .. ".factionbar.frame.border", factionbar.context) -- background frame for the bar
factionbar.text         = UI.CreateFrame("Text", addon.identifier .. ".factionbar.text", factionbar.context)          -- the text on the bar
-- UI elements: tooltip
local tooltip   = {}                                                                               -- "container" for tooltip
tooltip.context = UI.CreateContext(addon.identifier .. ".tooltip.context")                         -- context for the mouseover tooltip
tooltip.frame   = UI.CreateFrame("Texture", addon.identifier .. ".tooltip.frame", tooltip.context) -- tooltip frame
tooltip.text    = UI.CreateFrame("Text", addon.identifier .. ".tooltip.text", tooltip.context)     -- text in the tooltip
-- UI elements: drag&drop button
local movable = {}                                                              -- "container" for drag&drop button
movable.frame = UI.CreateFrame("Texture", addon.identifier, factionbar.context) -- button frame for moving the bar by drag&drop
-- UI elements: config dialog - common
local config         = {}                                                                                        -- "container" for common dialog elements
config.context       = UI.CreateContext(addon.identifier .. ".config.context")                                   -- context for config dialog
config.window        = UI.CreateFrame("RiftWindow", addon.identifier .. ".config.window", config.context)        -- main window for config dialog
config.background    = UI.CreateFrame("Texture", addon.identifier .. ".config.background", config.context)       -- main window for config dialog
config.button        = {}                                                                                        -- "container" for dialog buttons
config.button.close  = UI.CreateFrame("RiftButton", addon.identifier .. ".config.button.close", config.context)  -- close button (X) for config dialog
config.button.ok     = UI.CreateFrame("RiftButton", addon.identifier .. ".config.button.ok", config.context)     -- OK button for applying settings in config dialog
config.button.cancel = UI.CreateFrame("RiftButton", addon.identifier .. ".config.button.cancel", config.context) -- cancel button for config dialog
config.button.reset  = UI.CreateFrame("RiftButton", addon.identifier .. ".config.button.reset", config.context)  -- reset button for config dialog
config.button.descr  = UI.CreateFrame("Text", addon.identifier .. "config.button.descr", config.context)         -- description label for the buttons on mouseover
-- UI elements: config dialog - bar color
config.barcolor          = {}                                                                                            -- "container" for bar color elements
config.barcolor.descr    = UI.CreateFrame("Text", addon.identifier .. ".config.barcolor.descr", config.context)          -- text label for "bar color"
config.barcolor.descr.r  = UI.CreateFrame("Text", addon.identifier .. ".config.barcolor.descr.r", config.context)        -- text label for "R" (red)
config.barcolor.descr.g  = UI.CreateFrame("Text", addon.identifier .. ".config.barcolor.descr.g", config.context)        -- text label for "G" (green)
config.barcolor.descr.b  = UI.CreateFrame("Text", addon.identifier .. ".config.barcolor.descr.b", config.context)        -- text label for "B" (blue)
config.barcolor.slider   = {}                                                                                            -- "container" for bar color sliders
config.barcolor.slider.r = UI.CreateFrame("RiftSlider", addon.identifier .. ".config.barcolor.slider.r", config.context) -- slider for red
config.barcolor.slider.g = UI.CreateFrame("RiftSlider", addon.identifier .. ".config.barcolor.slider.g", config.context) -- slider for green
config.barcolor.slider.b = UI.CreateFrame("RiftSlider", addon.identifier .. ".config.barcolor.slider.b", config.context) -- slider for blue
config.barcolor.value    = {}                                                                                            -- "container" for bar color values
config.barcolor.value.r  = UI.CreateFrame("Text", addon.identifier .. ".config.barcolor.value.r", config.context)        -- text label for value of red slider
config.barcolor.value.g  = UI.CreateFrame("Text", addon.identifier .. ".config.barcolor.value.g", config.context)        -- text label for value of green slider
config.barcolor.value.b  = UI.CreateFrame("Text", addon.identifier .. ".config.barcolor.value.b", config.context)        -- text label for value of blue slider
config.barcolor.preview  = UI.CreateFrame("Frame", addon.identifier .. ".config.barcolor.preview", config.context)       -- preview frame for selected color
-- UI elements: config dialog - font color
config.fontcolor          = {}                                                                                             -- "container" for font color elements
config.fontcolor.descr    = UI.CreateFrame("Text", addon.identifier .. ".config.fontcolor.descr", config.context)          -- text label for "font color"
config.fontcolor.descr.r  = UI.CreateFrame("Text", addon.identifier .. ".config.fontcolor.descr.r", config.context)        -- text label for "R" (red)
config.fontcolor.descr.g  = UI.CreateFrame("Text", addon.identifier .. ".config.fontcolor.descr.g", config.context)        -- text label for "G" (green)
config.fontcolor.descr.b  = UI.CreateFrame("Text", addon.identifier .. ".config.fontcolor.descr.b", config.context)        -- text label for "B" (blue)
config.fontcolor.slider   = {}                                                                                             -- "container" for font color sliders
config.fontcolor.slider.r = UI.CreateFrame("RiftSlider", addon.identifier .. ".config.fontcolor.slider.r", config.context) -- slider for red
config.fontcolor.slider.g = UI.CreateFrame("RiftSlider", addon.identifier .. ".config.fontcolor.slider.g", config.context) -- slider for green
config.fontcolor.slider.b = UI.CreateFrame("RiftSlider", addon.identifier .. ".config.fontcolor.slider.b", config.context) -- slider for blue
config.fontcolor.value    = {}                                                                                             -- "container" for font color values
config.fontcolor.value.r  = UI.CreateFrame("Text", addon.identifier .. ".config.fontcolor.value.r", config.context)        -- text label for value of red slider
config.fontcolor.value.g  = UI.CreateFrame("Text", addon.identifier .. ".config.fontcolor.value.g", config.context)        -- text label for value of green slider
config.fontcolor.value.b  = UI.CreateFrame("Text", addon.identifier .. ".config.fontcolor.value.b", config.context)        -- text label for value of blue slider
config.fontcolor.preview  = UI.CreateFrame("Frame", addon.identifier .. ".config.fontcolor.preview", config.context)       -- preview frame for selected color
-- UI elements: config dialog - border color
config.bordercolor          = {}                                                                                               -- "container" for border color elements
config.bordercolor.descr    = UI.CreateFrame("Text", addon.identifier .. ".config.bordercolor.descr", config.context)          -- text label for "border color"
config.bordercolor.descr.r  = UI.CreateFrame("Text", addon.identifier .. ".config.bordercolor.descr.r", config.context)        -- text label for "R" (red)
config.bordercolor.descr.g  = UI.CreateFrame("Text", addon.identifier .. ".config.bordercolor.descr.g", config.context)        -- text label for "G" (green)
config.bordercolor.descr.b  = UI.CreateFrame("Text", addon.identifier .. ".config.bordercolor.descr.b", config.context)        -- text label for "B" (blue)
config.bordercolor.slider   = {}                                                                                               -- "container" for border color sliders
config.bordercolor.slider.r = UI.CreateFrame("RiftSlider", addon.identifier .. ".config.bordercolor.slider.r", config.context) -- slider for red
config.bordercolor.slider.g = UI.CreateFrame("RiftSlider", addon.identifier .. ".config.bordercolor.slider.g", config.context) -- slider for green
config.bordercolor.slider.b = UI.CreateFrame("RiftSlider", addon.identifier .. ".config.bordercolor.slider.b", config.context) -- slider for blue
config.bordercolor.value    = {}                                                                                               -- "container" for border color values
config.bordercolor.value.r  = UI.CreateFrame("Text", addon.identifier .. ".config.bordercolor.value.r", config.context)        -- text label for value of red slider
config.bordercolor.value.g  = UI.CreateFrame("Text", addon.identifier .. ".config.bordercolor.value.g", config.context)        -- text label for value of green slider
config.bordercolor.value.b  = UI.CreateFrame("Text", addon.identifier .. ".config.bordercolor.value.b", config.context)        -- text label for value of blue slider
config.bordercolor.preview  = UI.CreateFrame("Frame", addon.identifier .. ".config.bordercolor.preview", config.context)       -- preview frame for selected color
-- UI elements: config dialog - border transparency
config.bgtransparency          = {}                                                                                               -- "container" for border transparency elements
config.bgtransparency.descr    = UI.CreateFrame("Text", addon.identifier .. ".config.bgtransparency.descr", config.context)            -- text label for "border transparency"
config.bgtransparency.checkbox = UI.CreateFrame("RiftCheckbox", addon.identifier .. ".config.bgtransparency.checkbox", config.context) -- checkbox for border transparency
-- UI elements: config dialog - width
config.width        = {}                                                                                       -- "container" for bar width elements
config.width.descr  = UI.CreateFrame("Text", addon.identifier .. ".config.width.descr", config.context)        -- text label for "bar width"
config.width.slider = UI.CreateFrame("RiftSlider", addon.identifier .. ".config.width.slider", config.context) -- slider for width
config.width.value  = UI.CreateFrame("Text", addon.identifier .. ".config.width.value", config.context)        -- text label for value of width slider
-- UI elements: config dialog - height
config.height        = {}                                                                                        -- "container" for bar height elements
config.height.descr  = UI.CreateFrame("Text", addon.identifier .. ".config.height.descr", config.context)        -- text label for "bar height"
config.height.slider = UI.CreateFrame("RiftSlider", addon.identifier .. ".config.height.slider", config.context) -- slider for height
config.height.value  = UI.CreateFrame("Text", addon.identifier .. ".config.height.value", config.context)        -- text label for value of height slider
-- UI elements: config dialog - fontsize
config.fontsize        = {}                                                                                          -- "container" for fontsize elements
config.fontsize.descr  = UI.CreateFrame("Text", addon.identifier .. ".config.fontsize.descr", config.context)        -- text label for "fontsize"
config.fontsize.slider = UI.CreateFrame("RiftSlider", addon.identifier .. ".config.fontsize.slider", config.context) -- slider for fontsize
config.fontsize.value  = UI.CreateFrame("Text", addon.identifier .. ".config.fontsize.value", config.context)        -- text label for value of fontsize slider
-- UI elements: config dialog - transparency
config.transparency        = {}                                                                                              -- "container" for transparency elements
config.transparency.descr  = UI.CreateFrame("Text", addon.identifier .. ".config.transparency.descr", config.context)        -- text label for "transparency"
config.transparency.slider = UI.CreateFrame("RiftSlider", addon.identifier .. ".config.transparency.slider", config.context) -- slider for transparency
config.transparency.value  = UI.CreateFrame("Text", addon.identifier .. ".config.transparency.value", config.context)        -- text label for value of transparency slider
-- UI elements: config dialog - thousands separator
config.thousands          = {}                                                                                               -- "container" for thousands separator elements
config.thousands.descr    = UI.CreateFrame("Text", addon.identifier .. ".config.thousands.descr", config.context)            -- text label for "thousands separator"
config.thousands.checkbox = UI.CreateFrame("RiftCheckbox", addon.identifier .. ".config.thousands.checkbox", config.context) -- checkbox for thousands separator
config.thousands.value    = UI.CreateFrame("Text", addon.identifier .. ".config.thousands.value", config.context)            -- text label for value/example of thousands separator

-- default settings
local default          = {}                                       -- "container" (array) for default settings
default.maxwidth       = math.ceil(UIParent:GetWidth())           -- default maximum width of the UI
default.maxheight      = math.ceil(UIParent:GetHeight())          -- default maximum height of the UI
default.x              = math.ceil(default.maxwidth / 3)          -- default horizontal position (x) of the bar
default.y              = math.ceil(default.maxheight / 3)         -- default vertical position (y) of the bar
default.x1             = math.ceil((default.maxwidth / 2) - 250)  -- default horizontal position (x) of the config dialog
default.y1             = math.ceil((default.maxheight / 2) - 400) -- default vertical position (y) of the config dialog
default.width          = math.ceil(default.maxwidth / 3)          -- default width of the bar
default.height         = 12                                       -- default height of the bar / height=12 fits best for fontsize=13
default.fontsize       = 13                                       -- default fontsize of the text on the bar / fontsize=13 fits best for height=12
default.barcolor       = { r = 0, g = 1, b = 0 }                  -- default color of the bar (green)
default.bordercolor    = { r = 0, g = 0, b = 0 }                  -- default background-/bordercolor (black)
default.fontcolor      = { r = 1, g = 1, b = 1 }                  -- default color of the text (white)
default.transparency   = 0                                        -- default transparency of the whole bar (0=opaque, 100=transparent)
default.thousands      = false                                    -- default value for thousands separator
default.bgtransparency = false                                    -- default value for border transparency

-- settings
local x              = default.x              -- horizontal position (x) of the bar
local y              = default.y              -- vertical position (y) of the bar
local x1             = default.x              -- horizontal position (x) of the config dialog
local y1             = default.y              -- vertical position (y) of the config dialog
local width          = default.width          -- width of the bar
local height         = default.height         -- height of the bar
local fontsize       = default.fontsize       -- fontsize of the text on the bar
local barcolor       = default.barcolor       -- color of the bar
local bordercolor    = default.bordercolor    -- color of the bar
local fontcolor      = default.fontcolor      -- color of the text
local transparency   = default.transparency   -- transparency of the whole bar
local thousands      = default.thousands      -- value of thousands separator
local bgtransparency = default.bgtransparency -- value of border transparency
local padding        = 2                      -- border strength around the bar
local tooltipPadding = 20                     -- padding in mouseover tooltip

-- variables used for runtime
local mouseX                = 0      -- x-coordinate of mouse
local mouseY                = 0      -- y-coordinate of mouse
local nCurrent              = 0      -- notoriety of current faction; also used for calculating the "progress bar"
local nMaxCurrent           = 0      -- max. notoriety for current notoriety level (e.g. decorated=20000); also used for calculating the "progress bar"
local currentFaction        = ""     -- current faction (id) shown on the bar
local notorietyText         = ""     -- text shown on the bar
local notorietyTooltipText  = ""     -- text shown in the tooltip
local nList                 = {}     -- temporary variable for notorietySessionArray (notoriety list)
local notorietySessionArray = {}     -- "container" (array) for current session gains
local initialized           = false  -- controls the initialization of notorietySessionArray

-- formatting numbers with thousands separator
function format_thousand(num)
	local s = string.format("%d", math.floor(num))
	local pos = string.len(s) % 3
	if pos == 0 then
		pos = 3
	end
	return string.sub(s, 1, pos) .. string.gsub(string.sub(s, pos + 1), "(...)", text.separator .. "%1")
end

-- update notoriety list and bar
local function UpdateFaction()
	local n               = 0  -- amount of notoriety
	local nMax            = 0  -- max. amount of notoriety (e.g. decorated=20000)
	local percentage      = 0  -- percentage of current reached notoriety level (e.g. 5000/20000 -> 25%)
	local notorietyDescr  = "" -- name of the notoriety level (e.g. "Decorated")
	local notorietyGained = 0  -- the notoriety gained in current session for current faction (e.g. 10250/20000 (+250))
	local factionList = Inspect.Faction.List()
	if factionList and factionList ~= nil then
		factionDetails = Inspect.Faction.Detail(factionList)
		for id, detail in pairs(factionDetails) do
			n = detail.notoriety - 23000 -- neutral starts at 23000; notoriety at least neutral
			notorietyDescr = text.neutral
			nMax = 3000
			if n >= 3000 then -- at least friendly
				notorietyDescr = text.friendly
				n = n - 3000
				nMax = 10000
				if n >= 10000 then -- at least decorated
					notorietyDescr = text.decorated
					n = n - 10000
					nMax = 20000
					if n >= 20000 then -- at least honored
						notorietyDescr = text.honored
						n = n - 20000
						nMax = 35000
						if n >= 35000 then -- at least revered
							notorietyDescr = text.revered
							n = n - 35000
							nMax = 60000
							if n >= 60000 then -- at least glorified
								notorietyDescr = text.glorified
								n = n - 60000
								nMax = 90000
								if n >= 90000 then -- at least venerated (Patch 2.3)
									notorietyDescr = text.venerated
									n = n - 90000
									nMax = 120000
								end
							end
						end
					end
				end
			end
			if currentFaction == id or (factionPrevious[id] and factionPrevious[id] ~= nil and factionPrevious[id] > 0 and factionPrevious[id] ~= detail.notoriety) then
				nCurrent = n
				nMaxCurrent = nMax
				currentFaction = id
				percentage = math.ceil((1000 * n) / nMax) / 10
				if not notorietySessionArray[id] then
					nList = Inspect.Faction.List()
					notorietySessionArray = Inspect.Faction.Detail(nList)
					initialized = true
				end
				notorietyGained = detail.notoriety - notorietySessionArray[id].notoriety
				if thousands then
				notorietyText = detail.name .. ": " .. format_thousand(n) .. "/" .. format_thousand(nMax) .. " (" .. notorietyDescr .. ")"
				notorietyTooltipText = detail.name .. ":\n(" .. detail.categoryName .. ")\n" .. format_thousand(n) .. "/" .. format_thousand(nMax) .. " (" .. tostring(percentage) .. "%) " .. "(+" .. format_thousand(notorietyGained) .. ")\n" .. notorietyDescr
				else
					notorietyText = detail.name .. ": " .. tostring(n) .. "/" .. tostring(nMax) .. " (" .. notorietyDescr .. ")"
					notorietyTooltipText = detail.name .. ":\n(" .. detail.categoryName .. ")\n" .. tostring(n) .. "/" .. tostring(nMax) .. " (" .. tostring(percentage) .. "%) " .. "(+" .. tostring(notorietyGained) .. ")\n" .. notorietyDescr
				end
				factionbar.text:SetText(notorietyText)
				tooltip.text:SetText(notorietyTooltipText)
				factionbar.text:SetPoint("TOPLEFT", factionbar.frame.bar, "TOPLEFT", (width / 2) - (factionbar.text:GetWidth() / 2), (factionbar.frame.bar:GetHeight() / 2) - (factionbar.text:GetHeight() / 2))
				factionbar.frame.bar:SetWidth(math.ceil((n * (width - (2 * padding))) / nMax))
				factionbar.context:SetVisible(visible)
			else
				n = 0
				nMax = 0
				percentage = 0
			end
			factionPrevious[id] = detail.notoriety
		end
	else
		Command.Console.Display("general", true, "<font color='#ff0000'>" .. text.noFactionFoundError .. "</font>", true)
	end
end

-- perform a reset of all settings back to default ("/afb reset")
local function performReset()
	width          = default.width
	height         = default.height
	fontsize       = default.fontsize
	fontcolor      = default.fontcolor
	barcolor       = default.barcolor
	bordercolor    = default.bordercolor
	transparency   = default.transparency
	thousands      = default.thousands
	bgtransparency = default.bgtransparency
	currentFaction = ""
	x              = default.x
	y              = default.y
	x1             = default.x1
	y1             = default.y1
	locked         = true
	visible        = true
	mousedown      = false
	factionbar.frame.border:SetPoint("TOPLEFT", UIParent, "TOPLEFT", default.x, default.y)
	factionbar.frame.border:SetWidth(default.width)
	factionbar.frame.border:SetHeight(default.height)
	factionbar.frame.border:SetBackgroundColor(default.bordercolor.r, default.bordercolor.g, default.bordercolor.b, bgtransparency and 0 or 1)
	factionbar.frame.border:SetAlpha(1 - default.transparency)
	factionbar.frame.bar:SetHeight(default.height - (2 * padding))
	factionbar.frame.bar:SetBackgroundColor(default.barcolor.r, default.barcolor.g, default.barcolor.b)
	factionbar.frame.bar:SetAlpha(1 - default.transparency)
	factionbar.text:SetFontColor(default.fontcolor.r, default.fontcolor.g, default.fontcolor.b)
	factionbar.text:SetAlpha(1 - default.transparency)
	factionbar.text:SetFontSize(default.fontsize)
	movable.frame:SetVisible(false)
	config.window:SetPoint("TOPCENTER", UIParent, "TOPLEFT", x1, y1 - 30)
	Command.Console.Display("general", false, text.resettodefault, true)
	if nMaxCurrent == 0 then -- don't devide by zero, but this shouldn't happen normally
		factionbar.frame.bar:SetWidth(0)
	else
		factionbar.frame.bar:SetWidth(math.ceil((nCurrent * (default.width - (2 * padding))) / nMaxCurrent))
	end
	factionbar.text:SetPoint("TOPLEFT", factionbar.frame.bar, "TOPLEFT", (default.width / 2) - (factionbar.text:GetWidth() / 2), (factionbar.frame.bar:GetHeight() / 2) - (factionbar.text:GetHeight() / 2))
	repeat
		local result = collectgarbage("step", 1)
	until not result
end

local function changeWidth()
	factionbar.frame.border:SetWidth(width)
	factionbar.frame.border:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
	if nMaxCurrent == 0 then -- don't devide by zero, but this shouldn't happen normally
		factionbar.frame.bar:SetWidth(0)
	else
		factionbar.frame.bar:SetWidth(math.ceil((nCurrent * (width - (2 * padding))) / nMaxCurrent))
	end
	factionbar.text:SetPoint("TOPLEFT", factionbar.frame.bar, "TOPLEFT", (width / 2) - (factionbar.text:GetWidth() / 2), (factionbar.frame.bar:GetHeight() / 2) - (factionbar.text:GetHeight() / 2))
	repeat
		local result = collectgarbage("step", 1)
	until not result
end

local function changeHeight()
	factionbar.frame.border:SetHeight(height)
	factionbar.frame.border:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
	factionbar.frame.bar:SetHeight(height - (2 * padding))
	factionbar.text:SetPoint("TOPLEFT", factionbar.frame.bar, "TOPLEFT", (width / 2) - (factionbar.text:GetWidth() / 2), (factionbar.frame.bar:GetHeight() / 2) - (factionbar.text:GetHeight() / 2))
	repeat
		local result = collectgarbage("step", 1)
	until not result
end

-- handle slider change
function config.barcolor.slider.r.Event:SliderChange()
	config.barcolor.value.r:SetText(tostring(config.barcolor.slider.r:GetPosition()))
	config.barcolor.preview:SetBackgroundColor(config.barcolor.slider.r:GetPosition() / 255, config.barcolor.slider.g:GetPosition() / 255, config.barcolor.slider.b:GetPosition() / 255)
end

-- handle slider change
function config.barcolor.slider.g.Event:SliderChange()
	config.barcolor.value.g:SetText(tostring(config.barcolor.slider.g:GetPosition()))
	config.barcolor.preview:SetBackgroundColor(config.barcolor.slider.r:GetPosition() / 255, config.barcolor.slider.g:GetPosition() / 255, config.barcolor.slider.b:GetPosition() / 255)
end

-- handle slider change
function config.barcolor.slider.b.Event:SliderChange()
	config.barcolor.value.b:SetText(tostring(config.barcolor.slider.b:GetPosition()))
	config.barcolor.preview:SetBackgroundColor(config.barcolor.slider.r:GetPosition() / 255, config.barcolor.slider.g:GetPosition() / 255, config.barcolor.slider.b:GetPosition() / 255)
end

-- handle slider change
function config.fontcolor.slider.r.Event:SliderChange()
	config.fontcolor.value.r:SetText(tostring(config.fontcolor.slider.r:GetPosition()))
	config.fontcolor.preview:SetBackgroundColor(config.fontcolor.slider.r:GetPosition() / 255, config.fontcolor.slider.g:GetPosition() / 255, config.fontcolor.slider.b:GetPosition() / 255)
end

-- handle slider change
function config.fontcolor.slider.g.Event:SliderChange()
	config.fontcolor.value.g:SetText(tostring(config.fontcolor.slider.g:GetPosition()))
	config.fontcolor.preview:SetBackgroundColor(config.fontcolor.slider.r:GetPosition() / 255, config.fontcolor.slider.g:GetPosition() / 255, config.fontcolor.slider.b:GetPosition() / 255)
end

-- handle slider change
function config.fontcolor.slider.b.Event:SliderChange()
	config.fontcolor.value.b:SetText(tostring(config.fontcolor.slider.b:GetPosition()))
	config.fontcolor.preview:SetBackgroundColor(config.fontcolor.slider.r:GetPosition() / 255, config.fontcolor.slider.g:GetPosition() / 255, config.fontcolor.slider.b:GetPosition() / 255)
end

-- handle slider change
function config.bordercolor.slider.r.Event:SliderChange()
	config.bordercolor.value.r:SetText(tostring(config.bordercolor.slider.r:GetPosition()))
	config.bordercolor.preview:SetBackgroundColor(config.bordercolor.slider.r:GetPosition() / 255, config.bordercolor.slider.g:GetPosition() / 255, config.bordercolor.slider.b:GetPosition() / 255)
end

-- handle slider change
function config.bordercolor.slider.g.Event:SliderChange()
	config.bordercolor.value.g:SetText(tostring(config.bordercolor.slider.g:GetPosition()))
	config.bordercolor.preview:SetBackgroundColor(config.bordercolor.slider.r:GetPosition() / 255, config.bordercolor.slider.g:GetPosition() / 255, config.bordercolor.slider.b:GetPosition() / 255)
end

-- handle slider change
function config.bordercolor.slider.b.Event:SliderChange()
	config.bordercolor.value.b:SetText(tostring(config.bordercolor.slider.b:GetPosition()))
	config.bordercolor.preview:SetBackgroundColor(config.bordercolor.slider.r:GetPosition() / 255, config.bordercolor.slider.g:GetPosition() / 255, config.bordercolor.slider.b:GetPosition() / 255)
end

-- handle slider change for width
function config.width.slider.Event:SliderChange()
	if config.width.slider:GetPosition() / 5 ~= math.ceil(config.width.slider:GetPosition() / 5) and config.width.slider:GetPosition() < default.maxwidth then
		config.width.slider:SetPosition(math.ceil(config.width.slider:GetPosition() / 5) * 5)
	end
	config.width.value:SetText(tostring(config.width.slider:GetPosition()))
end

-- handle slider change for height
function config.height.slider.Event:SliderChange()
	config.height.value:SetText(tostring(config.height.slider:GetPosition()))
end

-- handle slider change for fontsize
function config.fontsize.slider.Event:SliderChange()
	config.fontsize.value:SetText(tostring(config.fontsize.slider:GetPosition()))
end

-- handle slider change for transparency
function config.transparency.slider.Event:SliderChange()
	if config.transparency.slider:GetPosition() / 5 ~= math.ceil(config.transparency.slider:GetPosition() / 5) then
		config.transparency.slider:SetPosition(math.ceil(config.transparency.slider:GetPosition() / 5) * 5)
	end
	config.transparency.value:SetText(tostring(config.transparency.slider:GetPosition()) .. "%")
end

-- close (hide) config dialog without saving settings
function config.button.close.Event:LeftClick()
	config.context:SetVisible(false)
	repeat
		local result = collectgarbage("step", 1)
	until not result
end

-- close (hide) config dialog without saving settings
function config.button.cancel.Event:LeftClick()
	config.context:SetVisible(false)
	repeat
		local result = collectgarbage("step", 1)
	until not result
end

-- reset all settings back to default
function config.button.reset.Event:LeftClick()
	config.barcolor.slider.r:SetPosition(default.barcolor.r * 255)
	config.barcolor.slider.g:SetPosition(default.barcolor.g * 255)
	config.barcolor.slider.b:SetPosition(default.barcolor.b * 255)
	config.fontcolor.slider.r:SetPosition(default.fontcolor.r * 255)
	config.fontcolor.slider.g:SetPosition(default.fontcolor.g * 255)
	config.fontcolor.slider.b:SetPosition(default.fontcolor.b * 255)
	config.bordercolor.slider.r:SetPosition(default.bordercolor.r * 255)
	config.bordercolor.slider.g:SetPosition(default.bordercolor.g * 255)
	config.bordercolor.slider.b:SetPosition(default.bordercolor.b * 255)
	config.width.slider:SetPosition(default.width)
	config.height.slider:SetPosition(default.height)
	config.fontsize.slider:SetPosition(default.fontsize)
	config.transparency.slider:SetPosition(default.transparency * 100)
	config.thousands.checkbox:SetChecked(default.thousands)
	config.bgtransparency.checkbox:SetChecked(default.bgtransparency)
	performReset()
	UpdateFaction()
	repeat
		local result = collectgarbage("step", 1)
	until not result
end

-- apply new settings and close (hide) config dialog
function config.button.ok.Event:LeftClick()
	barcolor       = { r = config.barcolor.slider.r:GetPosition() / 255, g = config.barcolor.slider.g:GetPosition() / 255, b = config.barcolor.slider.b:GetPosition() / 255 }
	bordercolor    = { r = config.bordercolor.slider.r:GetPosition() / 255, g = config.bordercolor.slider.g:GetPosition() / 255, b = config.bordercolor.slider.b:GetPosition() / 255 }
	fontcolor      = { r = config.fontcolor.slider.r:GetPosition() / 255, g = config.fontcolor.slider.g:GetPosition() / 255, b = config.fontcolor.slider.b:GetPosition() / 255 }
	transparency   = config.transparency.slider:GetPosition() / 100
	thousands      = config.thousands.checkbox:GetChecked()
	bgtransparency = config.bgtransparency.checkbox:GetChecked()
	width          = config.width.slider:GetPosition()
	height         = config.height.slider:GetPosition()
	fontsize       = config.fontsize.slider:GetPosition()
	factionbar.frame.bar:SetBackgroundColor(barcolor.r, barcolor.g, barcolor.b)
	factionbar.frame.border:SetBackgroundColor(bordercolor.r, bordercolor.g, bordercolor.b, bgtransparency and 0 or 1)
	factionbar.text:SetFontColor(fontcolor.r, fontcolor.g, fontcolor.b)
	factionbar.text:SetFontSize(fontsize)
	factionbar.frame.bar:SetAlpha(1 - transparency)
	factionbar.frame.border:SetAlpha(1 - transparency)
	factionbar.text:SetAlpha(1 - transparency)
	changeWidth()
	changeHeight()
	config.context:SetVisible(false)
	UpdateFaction()
	repeat
		local result = collectgarbage("step", 1)
	until not result
end

-- show mini tooltip/notice in config dialog
function config.button.ok.Event:MouseIn()
	config.button.descr:SetText(text.okdescr)
end

-- hide mini tooltip/notice in config dialog
function config.button.ok.Event:MouseOut()
	config.button.descr:SetText("")
end

-- show mini tooltip/notice in config dialog
function config.button.cancel.Event:MouseIn()
	config.button.descr:SetText(text.canceldescr)
end

-- hide mini tooltip/notice in config dialog
function config.button.cancel.Event:MouseOut()
	config.button.descr:SetText("")
end

-- show mini tooltip/notice in config dialog
function config.button.reset.Event:MouseIn()
	config.button.descr:SetText(text.inforeset)
end

-- hide mini tooltip/notice in config dialog
function config.button.reset.Event:MouseOut()
	config.button.descr:SetText("")
end

-- show mini tooltip/notice in config dialog
function config.button.close.Event:MouseIn()
	config.button.descr:SetText(text.canceldescr)
end

-- hide mini tooltip/notice in config dialog
function config.button.close.Event:MouseOut()
	config.button.descr:SetText("")
end

-- toggle checkbox changed of thousands separator checkbox
function config.thousands.checkbox.Event:CheckboxChange()
	if config.thousands.checkbox:GetChecked() then
		config.thousands.value:SetText(format_thousand(12345))
	else
		config.thousands.value:SetText("12345")
	end
end

-- shows the config dialog with actual values
local function ShowConfigDialog()
	config.barcolor.slider.r:SetPosition(math.floor(barcolor.r * 255))
	config.barcolor.slider.g:SetPosition(math.floor(barcolor.g * 255))
	config.barcolor.slider.b:SetPosition(math.floor(barcolor.b * 255))
	config.fontcolor.slider.r:SetPosition(math.floor(fontcolor.r * 255))
	config.fontcolor.slider.g:SetPosition(math.floor(fontcolor.g * 255))
	config.fontcolor.slider.b:SetPosition(math.floor(fontcolor.b * 255))
	config.bordercolor.slider.r:SetPosition(math.floor(bordercolor.r * 255))
	config.bordercolor.slider.g:SetPosition(math.floor(bordercolor.g * 255))
	config.bordercolor.slider.b:SetPosition(math.floor(bordercolor.b * 255))
	config.width.slider:SetPosition(width)
	config.height.slider:SetPosition(height)
	config.fontsize.slider:SetPosition(fontsize)
	config.transparency.slider:SetPosition(math.floor(transparency * 100))
	config.thousands.checkbox:SetChecked(thousands)
	config.bgtransparency.checkbox:SetChecked(bgtransparency)
	config.context:SetVisible(true)
end

-- toggle locked/movable
function movable.frame.Event:RightClick()
	factionbar.frame.border.Event:RightClick()
end

-- toggle locked/movable
function factionbar.frame.border.Event:RightClick()
	if locked then
		locked = false
		factionbar.frame.border:SetBackgroundColor(0, 0.5, 1, 1)
		visible=factionbar.frame.bar:GetVisible()
		factionbar.frame.bar:SetVisible(false)
		movable.frame:SetVisible(true)
		Command.Console.Display("general", false, text.movable, true)
	else
		locked = true
		factionbar.frame.border:SetBackgroundColor(bordercolor.r, bordercolor.g, bordercolor.b, bgtransparency and 0 or 1)
		factionbar.frame.bar:SetVisible(visible)
		movable.frame:SetVisible(false)
		Command.Console.Display("general", false, text.locked, true)
	end
	repeat
		local result = collectgarbage("step", 1)
	until not result
end

-- show tooltip
function factionbar.frame.border.Event:MouseIn()
	local mouseData = Inspect.Mouse()
	if notorietyTooltipText ~= "" and notorietyTooltipText ~= nil then
		local yOffset = 0
		local xOffset = 0
		tooltip.text:SetText(notorietyTooltipText)
		tooltip.frame:SetWidth(tooltip.text:GetWidth() + (2 * tooltipPadding))
		tooltip.frame:SetHeight(tooltip.text:GetHeight() + (2 * tooltipPadding))
		if mouseData.y <= tooltip.frame:GetHeight() then
			yOffset = tooltip.frame:GetHeight()
		end
		if mouseData.x >= default.maxwidth - tooltip.frame:GetWidth() then
			xOffset = -tooltip.frame:GetWidth()
		end
		tooltip.frame:SetPoint("BOTTOMLEFT", UIParent, "TOPLEFT", mouseData.x + xOffset, mouseData.y + yOffset)
		tooltip.context:SetVisible(true)
	end
end

-- hide tooltip
function factionbar.frame.border.Event:MouseOut()
	tooltip.context:SetVisible(false)
end

-- update tooltip/tooltip position when moving mouse
function factionbar.frame.border.Event:MouseMove()
	if notorietyTooltipText ~= "" and notorietyTooltipText ~= nil then
		local mouseData = Inspect.Mouse()
		local yOffset = 0
		local xOffset = 0
		if mouseData.y <= tooltip.frame:GetHeight() then
			yOffset = tooltip.frame:GetHeight()
		end
		if mouseData.x >= default.maxwidth - tooltip.frame:GetWidth() then
			xOffset = -tooltip.frame:GetWidth()
		end
		tooltip.frame:SetPoint("BOTTOMLEFT", UIParent, "TOPLEFT", mouseData.x + xOffset, mouseData.y + yOffset)
	end
end

-- show config dialog on middle-click on factionbar
function factionbar.frame.border.Event:MiddleClick()
	if locked == false then
		factionbar.frame.border.Event:RightClick()
	end
	ShowConfigDialog()
	repeat
		local result = collectgarbage("step", 1)
	until not result
end

-- handle left-click for drag&drop of bar
function movable.frame.Event:LeftDown()
	mousedown = true
end

-- handle left-click for drag&drop of config dialog
function config.background.Event:LeftDown()
	mousedown = true
end

-- handle left-click for drag&drop of config dialog
function config.window.Event:LeftDown()
	mousedown = true
end

-- handle mouse moving by drag&drop of bar
function movable.frame.Event:MouseMove()
	if mousedown and locked == false then
		local mouseData = Inspect.Mouse()
		x = math.ceil(mouseData.x)
		y = math.ceil(mouseData.y)
		factionbar.frame.border:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
	end
end

-- handle mouse moving by drag&drop of config dialog
function config.background.Event:MouseMove()
	if mousedown then
		local mouseData = Inspect.Mouse()
		x1 = math.ceil(mouseData.x)
		y1 = math.ceil(mouseData.y)
		config.window:SetPoint("TOPCENTER", UIParent, "TOPLEFT", x1, y1 - 30)
	end
end

-- handle mouse moving by drag&drop of config dialog
function config.window.Event:MouseMove()
	if mousedown then
		local mouseData = Inspect.Mouse()
		x1 = math.ceil(mouseData.x)
		y1 = math.ceil(mouseData.y)
		config.window:SetPoint("TOPCENTER", UIParent, "TOPLEFT", x1, y1 - 30)
	end
end

-- handle end of drag&drop of bar
function movable.frame.Event:LeftUp()
	mousedown = false
end

-- handle end of drag&drop of config dialog
function config.background.Event:LeftUp()
	mousedown = false
end

-- handle end of drag&drop of config dialog
function config.window.Event:LeftUp()
	mousedown = false
end

-- event raised when notoriety is gained
function OnNotoriety(handle, notoriety)
	if not initialized then
		nList = Inspect.Faction.List()
		notorietySessionArray = Inspect.Faction.Detail(nList)
		initialized = true
	end
	UpdateFaction()
end

-- check console commands and parameters
local function GetConsoleCommand(handle, args)
	local params = {}
	local numparams = 0
	for token in string.gmatch(args, "[^%s]+") do
		numparams = numparams + 1
		params[numparams] = token
	end
	if params[1] == "toggle" then
		-- toggle locked/movable
		factionbar.frame.border.Event:RightClick()
	elseif params[1] == "config" then
		-- show config dialog
		ShowConfigDialog()
	elseif params[1] == "reset" then
		-- reset size, position and colors back to default
		performReset()
	else
		--print help/usage
		local msg = "<u>" .. addon.identifier .. " " .. addon.toc.Version .. ":</u>\n"
		msg = msg .. text.usage .. "\n"
		msg = msg .. "<font color='#ffff00'>/afb</font> - " .. text.help .. "\n"
		msg = msg .. "<font color='#ffff00'>/afb config</font> - " .. text.infoconfig .. "\n"
		msg = msg .. "<font color='#ffff00'>/afb toggle</font> - " .. text.toggle .. "\n"
		msg = msg .. "<font color='#ffff00'>/afb reset</font> - " .. text.inforeset
		Command.Console.Display("general", true, msg, true)
	end
	repeat
		local result = collectgarbage("step", 1)
	until not result
end

-- save config to SavedVariables
local function SaveConfig(handle, value)
	if value ~= addon.identifier or value == nil then
		return
	end

	afbConfig.x              = x
	afbConfig.y              = y
	afbConfig.x1             = x1
	afbConfig.y1             = y1
	afbConfig.width          = width
	afbConfig.height         = height
	afbConfig.fontsize       = fontsize
	afbConfig.barcolor       = barcolor
	afbConfig.bordercolor    = bordercolor
	afbConfig.fontcolor      = fontcolor
	afbConfig.transparency   = transparency
	afbConfig.thousands      = thousands
	afbConfig.bgtransparency = bgtransparency
	afbConfig.lastFaction    = currentFaction
end

-- load config from SavedVariables
local function LoadConfig(handle, value)
	if value ~= addon.identifier or value == nil then
		return
	end

	afbConfig      = afbConfig or {}
	x              = afbConfig.x or default.x
	y              = afbConfig.y or default.y
	x1             = afbConfig.x1 or default.x1
	y1             = afbConfig.y1 or default.y1
	width          = afbConfig.width or default.width
	height         = afbConfig.height or default.height
	fontsize       = afbConfig.fontsize or default.fontsize
	barcolor       = afbConfig.barcolor or default.barcolor
	bordercolor    = afbConfig.bordercolor or default.bordercolor
	fontcolor      = afbConfig.fontcolor or default.fontcolor
	transparency   = afbConfig.transparency or default.transparency
	thousands      = afbConfig.thousands or default.thousands
	bgtransparency = afbConfig.bgtransparency or default.bgtransparency
	currentFaction = afbConfig.lastFaction or ""
end

-- create UI (set size, position, etc.)
local function CreateUI(handle, value)
	if value ~= addon.identifier or value == nil then
		return
	end

	local player=Inspect.Unit.Detail("player")

	factionbar.context:SetVisible(visible)
	tooltip.context:SetVisible(visible)
	movable.frame:SetVisible(visible)
	config.context:SetVisible(visible)

	-- init drag&drop button
	movable.frame:SetHeight(30)
	movable.frame:SetWidth(30)
	movable.frame:SetLayer(4)
	movable.frame:SetTexture(addon.identifier, "img/movable.png")
	movable.frame:SetPoint("CENTER", factionbar.frame.border, "TOPLEFT", 0, 0)

	-- init faction bar
	factionbar.frame.border:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
	factionbar.frame.border:SetWidth(width)
	factionbar.frame.border:SetBackgroundColor(bordercolor.r, bordercolor.g, bordercolor.b, bgtransparency and 0 or 1)
	factionbar.frame.border:SetAlpha(1 - transparency)
	factionbar.frame.border:SetLayer(1)
	factionbar.frame.bar:SetPoint("TOPLEFT", factionbar.frame.border, "TOPLEFT", padding, padding)
	factionbar.frame.bar:SetBackgroundColor(barcolor.r, barcolor.g, barcolor.b)
	factionbar.frame.bar:SetAlpha(1 - transparency)
	factionbar.frame.bar:SetLayer(2)
	factionbar.text:SetFontColor(fontcolor.r, fontcolor.g, fontcolor.b)
	factionbar.text:SetAlpha(1 - transparency)
	factionbar.text:SetFontSize(fontsize)
	factionbar.text:SetText("")
	factionbar.text:SetLayer(3)
	factionbar.frame.border:SetHeight(height)
	factionbar.frame.border:SetWidth(width)
	factionbar.frame.bar:SetHeight(height - (2 * padding))
	factionbar.frame.bar:SetWidth(0)
	factionbar.text:SetPoint("TOPLEFT", factionbar.frame.bar, "TOPLEFT", (width / 2) - (factionbar.text:GetWidth() / 2), (factionbar.frame.bar:GetHeight() / 2) - (factionbar.text:GetHeight() / 2))

	-- init tooltip
	tooltip.frame:SetTexture("Rift", "ItemToolTip_I75.dds")
	tooltip.frame:SetLayer(1)
	tooltip.text:SetPoint("TOPLEFT", tooltip.frame, "TOPLEFT", tooltipPadding, tooltipPadding)
	tooltip.text:SetFontColor(1, 1, 1)
	tooltip.text:SetFontSize(13)
	tooltip.text:SetText("")
	tooltip.text:SetLayer(2)

	-- init config dialog window
	config.window:SetPoint("TOPCENTER", UIParent, "TOPLEFT", x1, y1 - 30)
	config.window:SetTitle(addon.identifier .. " " .. addon.toc.Version .. " - " .. text.configheader)
	config.window:SetController("border")
	config.window:SetWidth(500)
	config.window:SetHeight(520)
	config.window:SetLayer(1)
	config.background:SetPoint("TOPLEFT", config.window, "TOPLEFT", 10, 10)
	if player.alliance == "defiant" then
		config.background:SetTexture("Rift", "Guild_Defiant_bg.png.dds")
	elseif player.alliance == "guardian" then
		config.background:SetTexture("Rift", "Guild_Guardian_bg.png.dds")
	end
	config.background:SetWidth(480)
	config.background:SetHeight(470)
	config.background:SetLayer(2)
	config.button.close:SetPoint("TOPRIGHT", config.window, "TOPRIGHT", -10, 15)
	config.button.close:SetSkin("close")
	config.button.close:SetLayer(3)
	config.button.ok:SetPoint("BOTTOMLEFT", config.window, "BOTTOMLEFT", 30, -20)
	config.button.ok:SetSkin("default")
	config.button.ok:SetText(text.ok)
	config.button.ok:SetLayer(3)
	config.button.cancel:SetPoint("BOTTOMRIGHT", config.window, "BOTTOMRIGHT", -30, -20)
	config.button.cancel:SetSkin("default")
	config.button.cancel:SetText(text.cancel)
	config.button.cancel:SetLayer(3)
	config.button.reset:SetPoint("BOTTOMCENTER", config.window, "BOTTOMCENTER", 0, -20)
	config.button.reset:SetSkin("default")
	config.button.reset:SetText(text.reset)
	config.button.reset:SetLayer(3)
	config.button.descr:SetPoint("BOTTOMCENTER", config.window, "BOTTOMCENTER", 0, -50)
	config.button.descr:SetFontSize(12)
	config.button.descr:SetText("")
	config.button.descr:SetLayer(3)

	-- init config dialog window: bar color elements
	config.barcolor.descr:SetPoint("TOPLEFT", config.window, "TOPLEFT", 25, 55)
	config.barcolor.descr:SetText(text.barcolor)
	config.barcolor.descr:SetFontSize(14)
	config.barcolor.descr:SetLayer(3)
	config.barcolor.descr.r:SetPoint("TOPLEFT", config.window, "TOPLEFT", 120, 55)
	config.barcolor.descr.r:SetFontColor(255, 0, 0)
	config.barcolor.descr.r:SetText("R:")
	config.barcolor.descr.r:SetFontSize(14)
	config.barcolor.descr.r:SetLayer(3)
	config.barcolor.descr.g:SetPoint("TOPLEFT", config.window, "TOPLEFT", 120, 75)
	config.barcolor.descr.g:SetFontColor(0, 255, 0)
	config.barcolor.descr.g:SetText("G:")
	config.barcolor.descr.g:SetFontSize(14)
	config.barcolor.descr.g:SetLayer(3)
	config.barcolor.descr.b:SetPoint("TOPLEFT", config.window, "TOPLEFT", 120, 95)
	config.barcolor.descr.b:SetFontColor(0, 0, 255)
	config.barcolor.descr.b:SetText("B:")
	config.barcolor.descr.b:SetFontSize(14)
	config.barcolor.descr.b:SetLayer(3)
	config.barcolor.slider.r:SetPoint("TOPLEFT", config.window, "TOPLEFT", 150, 60)
	config.barcolor.slider.r:SetRange(0, 255)
	config.barcolor.slider.r:SetHeight(15)
	config.barcolor.slider.r:SetLayer(3)
	config.barcolor.slider.g:SetPoint("TOPLEFT", config.window, "TOPLEFT", 150, 80)
	config.barcolor.slider.g:SetRange(0, 255)
	config.barcolor.slider.g:SetHeight(15)
	config.barcolor.slider.g:SetLayer(3)
	config.barcolor.slider.b:SetPoint("TOPLEFT", config.window, "TOPLEFT", 150, 100)
	config.barcolor.slider.b:SetRange(0, 255)
	config.barcolor.slider.b:SetHeight(15)
	config.barcolor.slider.b:SetLayer(3)
	config.barcolor.value.r:SetPoint("TOPLEFT", config.window, "TOPLEFT", 400, 55)
	config.barcolor.value.r:SetFontSize(14)
	config.barcolor.value.r:SetLayer(3)
	config.barcolor.value.g:SetPoint("TOPLEFT", config.window, "TOPLEFT", 400, 75)
	config.barcolor.value.g:SetFontSize(14)
	config.barcolor.value.g:SetLayer(3)
	config.barcolor.value.b:SetPoint("TOPLEFT", config.window, "TOPLEFT", 400, 95)
	config.barcolor.value.b:SetFontSize(14)
	config.barcolor.value.b:SetLayer(3)
	config.barcolor.preview:SetPoint("TOPLEFT", config.window, "TOPLEFT", 30, 80)
	config.barcolor.preview:SetWidth(60)
	config.barcolor.preview:SetHeight(30)
	config.barcolor.preview:SetLayer(3)

	-- init config dialog window: font color elements
	config.fontcolor.descr:SetPoint("TOPLEFT", config.window, "TOPLEFT", 25, 135)
	config.fontcolor.descr:SetText(text.fontcolor)
	config.fontcolor.descr:SetFontSize(14)
	config.fontcolor.descr:SetLayer(3)
	config.fontcolor.descr.r:SetPoint("TOPLEFT", config.window, "TOPLEFT", 120, 135)
	config.fontcolor.descr.r:SetFontColor(255, 0, 0)
	config.fontcolor.descr.r:SetText("R:")
	config.fontcolor.descr.r:SetFontSize(14)
	config.fontcolor.descr.r:SetLayer(3)
	config.fontcolor.descr.g:SetPoint("TOPLEFT", config.window, "TOPLEFT", 120, 155)
	config.fontcolor.descr.g:SetFontColor(0, 255, 0)
	config.fontcolor.descr.g:SetText("G:")
	config.fontcolor.descr.g:SetFontSize(14)
	config.fontcolor.descr.g:SetLayer(3)
	config.fontcolor.descr.b:SetPoint("TOPLEFT", config.window, "TOPLEFT", 120, 175)
	config.fontcolor.descr.b:SetFontColor(0, 0, 255)
	config.fontcolor.descr.b:SetText("B:")
	config.fontcolor.descr.b:SetFontSize(14)
	config.fontcolor.descr.b:SetLayer(3)
	config.fontcolor.slider.r:SetPoint("TOPLEFT", config.window, "TOPLEFT", 150, 140)
	config.fontcolor.slider.r:SetRange(0, 255)
	config.fontcolor.slider.r:SetHeight(15)
	config.fontcolor.slider.r:SetLayer(3)
	config.fontcolor.slider.g:SetPoint("TOPLEFT", config.window, "TOPLEFT", 150, 160)
	config.fontcolor.slider.g:SetRange(0, 255)
	config.fontcolor.slider.g:SetHeight(15)
	config.fontcolor.slider.g:SetLayer(3)
	config.fontcolor.slider.b:SetPoint("TOPLEFT", config.window, "TOPLEFT", 150, 180)
	config.fontcolor.slider.b:SetRange(0, 255)
	config.fontcolor.slider.b:SetHeight(15)
	config.fontcolor.slider.b:SetLayer(3)
	config.fontcolor.value.r:SetPoint("TOPLEFT", config.window, "TOPLEFT", 400, 135)
	config.fontcolor.value.r:SetFontSize(14)
	config.fontcolor.value.r:SetLayer(3)
	config.fontcolor.value.g:SetPoint("TOPLEFT", config.window, "TOPLEFT", 400, 155)
	config.fontcolor.value.g:SetFontSize(14)
	config.fontcolor.value.g:SetLayer(3)
	config.fontcolor.value.b:SetPoint("TOPLEFT", config.window, "TOPLEFT", 400, 175)
	config.fontcolor.value.b:SetFontSize(14)
	config.fontcolor.value.b:SetLayer(3)
	config.fontcolor.preview:SetPoint("TOPLEFT", config.window, "TOPLEFT", 30, 160)
	config.fontcolor.preview:SetWidth(60)
	config.fontcolor.preview:SetHeight(30)
	config.fontcolor.preview:SetLayer(3)

	-- init config dialog window: border color elements
	config.bordercolor.descr:SetPoint("TOPLEFT", config.window, "TOPLEFT", 25, 215)
	config.bordercolor.descr:SetText(text.bordercolor)
	config.bordercolor.descr:SetFontSize(14)
	config.bordercolor.descr:SetLayer(3)
	config.bordercolor.descr.r:SetPoint("TOPLEFT", config.window, "TOPLEFT", 120, 215)
	config.bordercolor.descr.r:SetFontColor(255, 0, 0)
	config.bordercolor.descr.r:SetText("R:")
	config.bordercolor.descr.r:SetFontSize(14)
	config.bordercolor.descr.r:SetLayer(3)
	config.bordercolor.descr.g:SetPoint("TOPLEFT", config.window, "TOPLEFT", 120, 235)
	config.bordercolor.descr.g:SetFontColor(0, 255, 0)
	config.bordercolor.descr.g:SetText("G:")
	config.bordercolor.descr.g:SetFontSize(14)
	config.bordercolor.descr.g:SetLayer(3)
	config.bordercolor.descr.b:SetPoint("TOPLEFT", config.window, "TOPLEFT", 120, 255)
	config.bordercolor.descr.b:SetFontColor(0, 0, 255)
	config.bordercolor.descr.b:SetText("B:")
	config.bordercolor.descr.b:SetFontSize(14)
	config.bordercolor.descr.b:SetLayer(3)
	config.bordercolor.slider.r:SetPoint("TOPLEFT", config.window, "TOPLEFT", 150, 220)
	config.bordercolor.slider.r:SetRange(0, 255)
	config.bordercolor.slider.r:SetHeight(15)
	config.bordercolor.slider.r:SetLayer(3)
	config.bordercolor.slider.g:SetPoint("TOPLEFT", config.window, "TOPLEFT", 150, 240)
	config.bordercolor.slider.g:SetRange(0, 255)
	config.bordercolor.slider.g:SetHeight(15)
	config.bordercolor.slider.g:SetLayer(3)
	config.bordercolor.slider.b:SetPoint("TOPLEFT", config.window, "TOPLEFT", 150, 260)
	config.bordercolor.slider.b:SetRange(0, 255)
	config.bordercolor.slider.b:SetHeight(15)
	config.bordercolor.slider.b:SetLayer(3)
	config.bordercolor.value.r:SetPoint("TOPLEFT", config.window, "TOPLEFT", 400, 215)
	config.bordercolor.value.r:SetFontSize(14)
	config.bordercolor.value.r:SetLayer(3)
	config.bordercolor.value.g:SetPoint("TOPLEFT", config.window, "TOPLEFT", 400, 235)
	config.bordercolor.value.g:SetFontSize(14)
	config.bordercolor.value.g:SetLayer(3)
	config.bordercolor.value.b:SetPoint("TOPLEFT", config.window, "TOPLEFT", 400, 255)
	config.bordercolor.value.b:SetFontSize(14)
	config.bordercolor.value.b:SetLayer(3)
	config.bordercolor.preview:SetPoint("TOPLEFT", config.window, "TOPLEFT", 30, 240)
	config.bordercolor.preview:SetWidth(60)
	config.bordercolor.preview:SetHeight(30)
	config.bordercolor.preview:SetLayer(3)

	-- init config dialog window: border transparency elements
	config.bgtransparency.descr:SetPoint("TOPLEFT", config.window, "TOPLEFT", 150, 280)
	config.bgtransparency.descr:SetText(text.bgtransparency)
	config.bgtransparency.descr:SetFontSize(14)
	config.bgtransparency.descr:SetLayer(3)
	config.bgtransparency.checkbox:SetPoint("TOPLEFT", config.window, "TOPLEFT", 120, 280)
	config.bgtransparency.checkbox:SetHeight(18)
	config.bgtransparency.checkbox:SetLayer(3)

	-- init config dialog window: width elements
	config.width.descr:SetPoint("TOPLEFT", config.window, "TOPLEFT", 25, 320)
	config.width.descr:SetText(text.width)
	config.width.descr:SetFontSize(14)
	config.width.descr:SetLayer(3)
	config.width.slider:SetPoint("TOPLEFT", config.window, "TOPLEFT", 150, 325)
	config.width.slider:SetRange(200, default.maxwidth)
	config.width.slider:SetHeight(15)
	config.width.slider:SetLayer(3)
	config.width.value:SetPoint("TOPLEFT", config.window, "TOPLEFT", 400, 320)
	config.width.value:SetFontSize(14)
	config.width.value:SetLayer(3)

	-- init config dialog window: height elements
	config.height.descr:SetPoint("TOPLEFT", config.window, "TOPLEFT", 25, 345)
	config.height.descr:SetText(text.height)
	config.height.descr:SetFontSize(14)
	config.height.descr:SetLayer(3)
	config.height.slider:SetPoint("TOPLEFT", config.window, "TOPLEFT", 150, 350)
	config.height.slider:SetRange(10, 50)
	config.height.slider:SetHeight(15)
	config.height.slider:SetLayer(3)
	config.height.value:SetPoint("TOPLEFT", config.window, "TOPLEFT", 400, 345)
	config.height.value:SetFontSize(14)
	config.height.value:SetLayer(3)

	-- init config dialog window: fontsize elements
	config.fontsize.descr:SetPoint("TOPLEFT", config.window, "TOPLEFT", 25, 370)
	config.fontsize.descr:SetText(text.fontsize)
	config.fontsize.descr:SetFontSize(14)
	config.fontsize.descr:SetLayer(3)
	config.fontsize.slider:SetPoint("TOPLEFT", config.window, "TOPLEFT", 150, 375)
	config.fontsize.slider:SetRange(8, 20)
	config.fontsize.slider:SetHeight(15)
	config.fontsize.slider:SetLayer(3)
	config.fontsize.value:SetPoint("TOPLEFT", config.window, "TOPLEFT", 400, 370)
	config.fontsize.value:SetFontSize(14)
	config.fontsize.value:SetLayer(3)

	-- init config dialog window: transparency elements
	config.transparency.descr:SetPoint("TOPLEFT", config.window, "TOPLEFT", 25, 395)
	config.transparency.descr:SetText(text.transparency)
	config.transparency.descr:SetFontSize(14)
	config.transparency.descr:SetLayer(3)
	config.transparency.slider:SetPoint("TOPLEFT", config.window, "TOPLEFT", 150, 400)
	config.transparency.slider:SetRange(0, 100)
	config.transparency.slider:SetHeight(15)
	config.transparency.slider:SetLayer(3)
	config.transparency.value:SetPoint("TOPLEFT", config.window, "TOPLEFT", 400, 395)
	config.transparency.value:SetFontSize(14)
	config.transparency.value:SetLayer(3)

	-- init config dialog window: thousands separator elements
	config.thousands.descr:SetPoint("TOPLEFT", config.window, "TOPLEFT", 25, 420)
	config.thousands.descr:SetText(text.thousands)
	config.thousands.descr:SetFontSize(14)
	config.thousands.descr:SetLayer(3)
	config.thousands.checkbox:SetPoint("TOPLEFT", config.window, "TOPLEFT", 150, 420)
	config.thousands.checkbox:SetHeight(18)
	config.thousands.checkbox:SetLayer(3)
	config.thousands.value:SetPoint("TOPLEFT", config.window, "TOPLEFT", 400, 420)
	config.thousands.value:SetFontSize(14)
	config.thousands.value:SetLayer(3)

	visible=true
	factionbar.context:SetVisible(visible)

	UpdateFaction(notoriety) -- initially run a "faction scan"
	repeat
		local result = collectgarbage("step", 1)
	until not result
	print(addon.toc.Version .. " " .. text.loaded)
end

Command.Event.Attach(Event.Addon.SavedVariables.Load.End, LoadConfig, "Event.Addon.SavedVariables.Load.End.LoadConfig")
Command.Event.Attach(Event.Addon.SavedVariables.Save.Begin, SaveConfig, "Event.Addon.SavedVariables.Save.Begin.SaveConfig")
Command.Event.Attach(Event.Faction.Notoriety, OnNotoriety, "Event.Faction.Notoriety.OnNotoriety")
Command.Event.Attach(Command.Slash.Register("autofactionbar"), GetConsoleCommand, "Command.Slash.Register.GetConsoleCommand")
Command.Event.Attach(Command.Slash.Register("afb"), GetConsoleCommand, "Command.Slash.Register.GetConsoleCommand")
Command.Event.Attach(Event.Addon.Load.End, CreateUI, "Event.Addon.Load.End.CreateUI")
